package net.sf.doolin.util;

/**
 * Adapter between a subject type and a target type.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <S>
 *            Subject type
 * @param <T>
 *            Target type
 */
public interface Adapter<S, T> {

	/**
	 * Conversion from the subject to the target
	 * 
	 * @param subjectValue
	 *            Subject value
	 * @return Target value
	 */
	T convertSubjectToTarget(S subjectValue);

	/**
	 * Conversion from the target to the subject
	 * 
	 * @param targetValue
	 *            Target value
	 * @return Subject value
	 */
	S convertTargetToSubject(T targetValue);

}
