package net.sf.doolin.util;

import com.google.common.base.Function;

/**
 * This class performs a conversion by performing a copy using an
 * {@link ItemCopier}. The type of the target is identical to the one of the
 * source.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <T>
 *            Type of both the target and source.
 */
public class CopyFunction<T> implements Function<T, T> {

	private ItemCopier<T> copier = new SimpleItemCopier<T>();

	@Override
	public T apply(T source) {
		@SuppressWarnings("unchecked")
		Class<? extends T> cl = (Class<? extends T>) source.getClass();
		T t = Utils.newInstance(cl);
		this.copier.copy(source, t);
		return t;
	}

	/**
	 * Sets the copier to use (required)
	 * 
	 * @param copier
	 *            Copier implementation
	 */
	public void setCopier(ItemCopier<T> copier) {
		this.copier = copier;
	}

}
