package net.sf.doolin.util;

import com.google.common.base.Function;

/**
 * Converts a S object into a T object by calling a method on S.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <S>
 *            Type of the source object
 * @param <T>
 *            Type of the target object
 */
public class MethodFunction<S, T> implements Function<S, T> {

	private final String methodName;

	/**
	 * Constructor
	 * 
	 * @param methodName
	 *            Method name
	 */
	public MethodFunction(String methodName) {
		this.methodName = methodName;
	}

	/**
	 * Calls the method on the <code>source</code> object and casts the result
	 * to <code>&lt;T&gt;</code>.
	 */
	@Override
	public T apply(S source) {
		@SuppressWarnings("unchecked")
		T t = (T) Utils.callMethod(source, this.methodName);
		return t;
	}

	/**
	 * Returns the method name
	 * 
	 * @return Method name
	 */
	public String getMethodName() {
		return this.methodName;
	}

}
