/**
 * 
 */
package net.sf.doolin.util.expression;

import java.util.Locale;

/**
 * Token definition in an expression.
 * 
 * @author Damien Coraboeuf
 */
public interface Token {

	/**
	 * Adds the.
	 * 
	 * @param add
	 *            the add
	 */
	void add(Token add);

	/**
	 * Adds the text.
	 * 
	 * @param c
	 *            the c
	 * 
	 * @return the token
	 */
	Token addText(char c);

	/**
	 * Close.
	 */
	void close();

	/**
	 * Evaluate.
	 * 
	 * @param locale
	 *            Locale used for the evaluation
	 * @param context
	 *            Context for the evaluation
	 * @return Result of the evaluation
	 */
	Object evaluate(Locale locale, Object context);

	/**
	 * Gets the type.
	 * 
	 * @return the type
	 */
	TokenType getType();

	/**
	 * Checks if is closed.
	 * 
	 * @return true, if is closed
	 */
	boolean isClosed();

	/**
	 * Checks if the expression contains parts which can vary according to the
	 * model.
	 * 
	 * @return <code>true</code> if some parts of the expression are dependent
	 *         on the model.
	 */
	boolean isVariable();

	/**
	 * Support add.
	 * 
	 * @return true, if successful
	 */
	boolean supportAdd();

	/**
	 * Visits a the whole token tree with a visitor.
	 * 
	 * @param visitor
	 *            Visitor to apply for this token and each token underneath.
	 */
	void visitTokensWith(TokenVisitor visitor);
}