/**
 * 
 */
package net.sf.doolin.util.expression;

import java.util.Locale;

import net.sf.doolin.util.Utils;

/**
 * Token for a <code>@{...}</code> token.
 * 
 * @author Damien Coraboeuf
 * 
 */
public class ValueToken extends AbstractToken {

	private String propertyPath;

	private boolean bound = true;

	/**
	 * Instantiates a new value token.
	 */
	public ValueToken() {
		super(TokenType.VALUE);
		this.propertyPath = "";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void add(Token add) {
		throw new UnsupportedOperationException();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Token addText(char c) {
		if (c == '!') {
			this.bound = false;
			return null;
		} else {
			this.propertyPath += c;
			return null;
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Object evaluate(Locale locale, Object context) {
		Object value = Utils.getProperty(context, this.propertyPath);
		return value;
	}

	/**
	 * Returns the associated property path
	 * 
	 * @return Property path
	 */
	public String getPropertyPath() {
		return this.propertyPath;
	}

	/**
	 * Returns if the associated value must be bound with any change of the
	 * property value
	 * 
	 * @return <code>true</code> if the message must change whenever the
	 *         property changes
	 */
	public boolean isBound() {
		return this.bound;
	}

	@Override
	public boolean isVariable() {
		return this.bound;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean supportAdd() {
		return false;
	}

	@Override
	public void visitTokensWith(TokenVisitor visitor) {
		visitor.visitValueToken(this);
	}

}