package net.sf.doolin.util.factory;

import java.util.HashMap;
import java.util.Map;

import net.sf.doolin.util.Utils;

/**
 * Calls another data factory and initialises some of its properties.
 * 
 * @author Damien Coraboeuf
 * 
 * @param <T>
 *            Type of object to create
 */
public class InitialisingDataFactory<T> implements DataFactory<T> {

	private DataFactory<T> dataFactory = new IdentityDataFactory<T>();

	private Map<String, Object> initialisationMap = new HashMap<String, Object>();

	@Override
	public T create(Object context) {
		T t = this.dataFactory.create(context);
		for (Map.Entry<String, Object> entry : this.initialisationMap
				.entrySet()) {
			String propertyPath = entry.getKey();
			Object propertyValue = entry.getValue();
			Utils.setProperty(t, propertyPath, propertyValue);
		}
		return t;
	}

	/**
	 * Gets the factory to call first.
	 * 
	 * @return Data factory
	 */
	public DataFactory<T> getDataFactory() {
		return this.dataFactory;
	}

	/**
	 * Gets the property paths and their values
	 * 
	 * @return Map of property paths/values
	 */
	public Map<String, Object> getInitialisationMap() {
		return this.initialisationMap;
	}

	/**
	 * Sets the factory to call first.
	 * 
	 * @param dataFactory
	 *            Data factory
	 */
	public void setDataFactory(DataFactory<T> dataFactory) {
		this.dataFactory = dataFactory;
	}

	/**
	 * Sets the property paths and their values
	 * 
	 * @param initialisationMap
	 *            Map of property paths/values
	 */
	public void setInitialisationMap(Map<String, Object> initialisationMap) {
		this.initialisationMap = initialisationMap;
	}

}
